/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SimpleObservationTripleMarshaller.java,v deced55ab7da 2010/06/24 16:14:12 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation.semantic;

import javax.annotation.Nonnull;
import java.util.List;
import it.tidalwave.util.As;
import it.tidalwave.util.AsException;
import it.tidalwave.semantic.io.json.Triple;
import it.tidalwave.semantic.io.json.spi.impl.AsTripleMarshaller;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.Observer;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class SimpleObservationTripleMarshaller extends AsTripleMarshaller
  {
    @Override @Nonnull
    public List<Triple> marshal (final @Nonnull As entity)
      {
        final List<Triple> triples = super.marshal(entity);
        final Observation observation = (Observation)entity;
        triples.add(new Triple(observation, Vocabulary.RDF_ABOUT, Vocabulary.OBSERVATION));
        triples.add(new Triple(observation, Vocabulary.DC_DATE, observation.getDate()));
        triples.add(new Triple(observation, Vocabulary.MADE_AT, observation.getLocation()));
        triples.add(new Triple(observation, Vocabulary.DC_PUBLISHER, observation.getSource()));

        for (final Observer observer : observation.findObservers().results())
          {
            triples.add(new Triple(observation, Vocabulary.DC_CREATOR, observer));
          }

        for (final ObservationItem observationItem : observation.findObservationItems().results())
          {
            triples.add(new Triple(observation, Vocabulary.SKOS_NARROWER, observationItem));
          }

        try
          {
            triples.add(new Triple(observation.as(Media.class), Vocabulary.DC_SUBJECT, observation));
          }
        catch (AsException e)
          {
            // ok, no attachment
          }

        return triples;
      }
  }
