/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SimpleObservationTripleUnmarshaller.java,v deced55ab7da 2010/06/24 16:14:12 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation.semantic;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import it.tidalwave.util.As;
import it.tidalwave.util.DefaultDisplayable;
import it.tidalwave.util.DefaultIdentifiable;
import it.tidalwave.semantic.io.json.spi.TripleUnmarshallerSupport;
import it.tidalwave.semantic.io.json.impl.DehydratedTriple;
import it.tidalwave.bluebill.observation.Location;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.Observer;
import it.tidalwave.bluebill.observation.Source;
import it.tidalwave.bluebill.observation.simple.SimpleObservation;
import it.tidalwave.bluebill.observation.simple.SimpleObservationItem;
import it.tidalwave.bluebill.observation.simple.SimpleObservationSet;
import it.tidalwave.bluebill.observation.simple.SimpleObserver;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class SimpleObservationTripleUnmarshaller extends TripleUnmarshallerSupport
  {
    @Override @Nonnull
    public As unmarshal (final @Nonnull List<DehydratedTriple> triples, final @Nonnull Context context)
      {
        final SimpleObservationSet observationSet = (SimpleObservationSet)context.peek();
        final Date date = (Date)getLiteral(triples, Vocabulary.DC_DATE);
        final Location location = (Location)findEntity(context, triples, Vocabulary.MADE_AT);
        final Source source = (Source)findEntity(context, triples, Vocabulary.DC_PUBLISHER);
        final Set<Observer> observers = new HashSet<Observer>();
        final Set<ObservationItem> observationItems = new HashSet<ObservationItem>();
        final List<Object> extras = new ArrayList<Object>();
        extras.add(new DefaultIdentifiable(triples.get(0).getSubject()));

        final String displayName = (String)getLiteral(triples, Vocabulary.RDFS_LABEL);

        if (displayName != null)
          {
            extras.add(new DefaultDisplayable(displayName, "SimpleObservation"));
          }

        for (final DehydratedTriple triple : context.findTriples(Vocabulary.DC_SUBJECT, triples.get(0).getSubject()))
          {
            extras.add(context.find(triple.getSubject()));
          }

        final SimpleObservation observation = new SimpleObservation(observationSet, date, location, observers, observationItems, source, extras);
        context.push(observation);

        for (final DehydratedTriple triple : findTriplesWithPredicate(triples, Vocabulary.SKOS_NARROWER))
          {
            observationItems.add((SimpleObservationItem)context.find(triple.getObject()));
          }

        for (final DehydratedTriple triple : findTriplesWithPredicate(triples, Vocabulary.DC_CREATOR))
          {
            observers.add((SimpleObserver)context.find(triple.getObject()));
          }

        context.pop();

        return observation;
      }
  }
