/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TripleMarshallerFactoryImpl.java,v 96a070c6a0a0 2010/06/23 23:07:19 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.othermarshallers;

import it.tidalwave.semantic.io.json.spi.TripleMarshallerFactory;

import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.Map;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.As;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.semantic.io.json.spi.TripleMarshaller;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.bluebill.observation.Location;
import it.tidalwave.bluebill.observation.Observable;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.ObservationSet;
import it.tidalwave.bluebill.observation.Observer;
import it.tidalwave.bluebill.observation.Source;
import it.tidalwave.bluebill.observation.semantic.SimpleLocationTripleMarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObservableTripleMarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObservationItemTripleMarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObservationSetTripleMarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObservationTripleMarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObserverTripleMarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleSourceTripleMarshaller;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@ServiceProvider(service=TripleMarshallerFactory.class)
public class TripleMarshallerFactoryImpl implements TripleMarshallerFactory
  {
    private final Map<Class<?>, TripleMarshaller> tripleMarshallerMapByClass = new HashMap<Class<?>, TripleMarshaller>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TripleMarshallerFactoryImpl()
      {
        // IXME: discover with Lookup
//        tripleMarshallerMapByClass.put(FactSheet.class, new FactSheetTripleMarshaller());
        tripleMarshallerMapByClass.put(Media.class, new MediaTripleMarshaller());
        tripleMarshallerMapByClass.put(ObservationSet.class, new SimpleObservationSetTripleMarshaller());
        tripleMarshallerMapByClass.put(Observation.class, new SimpleObservationTripleMarshaller());
        tripleMarshallerMapByClass.put(ObservationItem.class, new SimpleObservationItemTripleMarshaller());
        tripleMarshallerMapByClass.put(Observable.class, new SimpleObservableTripleMarshaller());
        tripleMarshallerMapByClass.put(Observer.class, new SimpleObserverTripleMarshaller());
        tripleMarshallerMapByClass.put(Source.class, new SimpleSourceTripleMarshaller());
        tripleMarshallerMapByClass.put(Location.class, new SimpleLocationTripleMarshaller());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public TripleMarshaller findTripleMarshallerFor (final @Nonnull As object) // TODO: replace with object.as(TripleFactory)
      throws NotFoundException
      {
        final Class<?> clazz = object.getClass();

        for (final Entry<Class<?>, TripleMarshaller> entry : tripleMarshallerMapByClass.entrySet())
          {
            if (entry.getKey().isAssignableFrom(clazz))
              {
                return entry.getValue();
              }
          }

        throw new NotFoundException(clazz.getName());
      }
  }
