/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TripleUnmarshallerFactoryImpl.java,v e82a4eeab3f8 2010/06/24 17:43:51 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.othermarshallers;

import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.semantic.io.json.spi.TripleUnmarshaller;
import it.tidalwave.semantic.io.json.spi.TripleUnmarshallerFactory;
import it.tidalwave.semantic.io.json.spi.TripleUnmarshallerSupport;
import it.tidalwave.semantic.io.json.impl.DehydratedTriple;
import it.tidalwave.bluebill.observation.semantic.SimpleLocationTripleUnmarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObservableTripleUnmarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObservationItemTripleUnmarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObservationSetTripleUnmarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObservationTripleUnmarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleObserverTripleUnmarshaller;
import it.tidalwave.bluebill.observation.semantic.SimpleSourceTripleUnmarshaller;
import it.tidalwave.bluebill.observation.semantic.Vocabulary;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@ServiceProvider(service=TripleUnmarshallerFactory.class)
public class TripleUnmarshallerFactoryImpl implements TripleUnmarshallerFactory
  {
    private final Map<Id, TripleUnmarshallerSupport> factoryMapByType = new HashMap<Id, TripleUnmarshallerSupport>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TripleUnmarshallerFactoryImpl()
      {
        // IXME: discover with Lookup
        factoryMapByType.put(Vocabulary.OBSERVATION_SET, new SimpleObservationSetTripleUnmarshaller());
        factoryMapByType.put(Vocabulary.OBSERVATION, new SimpleObservationTripleUnmarshaller());
        factoryMapByType.put(Vocabulary.OBSERVATION_ITEM, new SimpleObservationItemTripleUnmarshaller());
        factoryMapByType.put(Vocabulary.OBSERVABLE, new SimpleObservableTripleUnmarshaller());
        factoryMapByType.put(Vocabulary.OBSERVER, new SimpleObserverTripleUnmarshaller());
        factoryMapByType.put(Vocabulary.DC_PUBLISHER, new SimpleSourceTripleUnmarshaller());
        factoryMapByType.put(Vocabulary.LOCATION, new SimpleLocationTripleUnmarshaller());
//        factoryMapByType.put(Media.MEDIA, new MediaTripleUnmarshaller());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public TripleUnmarshaller findTripleUnmarshallerFor (final @Nonnull List<DehydratedTriple> triples)
      throws NotFoundException
      {
        TripleUnmarshaller unmarshaller = null; // = new EntityTripleUnmarshaller();

        // Find the best factory - FIXME: you should pick the most specialized one, but where to find hierarchy info?
        for (final DehydratedTriple triple :triples)
          {
            TripleUnmarshallerSupport temp = null;

            if (triple.getPredicate().equals(Vocabulary.RDF_ABOUT))
              {
                temp = factoryMapByType.get(new Id((String) triple.getObject()));
              }
            else if (triple.getPredicate().equals(Vocabulary.DC_TYPE) && triple.getObject().toString().startsWith("http://purl.org/dc/dcmitype/"))
              {
                temp = new MediaTripleUnmarshaller();
              }
            
            if (temp != null)
              {
                unmarshaller = temp;  
              }
          }

        return NotFoundException.throwWhenNull(unmarshaller, "No TripleUnmarshaller for %s", triples);
      }
  }
